#!/bin/sh
#
# Backup Cisco configuration file to TFTP server plugin for Nagios
# Written by Garry W. Cook (garry@cookbros.net)
# Last Modified: 20040224
#
#
# Usage: ./check_cisco_config_backup <ios|catos> <timeout> <snmp_community> <device_ip> <filename> <tftp_server_ip>
#
#
# Description:
#
# This plugin will use the snmpset command to force a Cisco Device to
# upload its configuration file (IOS=running config) to a TFTP server.
#
#
# Output:
#
# When running the plugin, an OK state will be returned if the TFTP
# upload is successful. If the upload is unsuccessful, it will return
# a CRITICAL state. For IOS devices, if the running configuration is
# uploaded but not written to NVRAM, a WARNING state will be returned.
#
#
# Notes:
#
# If the TFTP server does not allow for file creation, then the file MUST
# EXIST on the server previous to using this plugin. You are responsible
# for correctly setting up your TFTP Server. This plugin knows nothing about
# the TFTP server other than its IP address.
# Please also keep in mind that '.cfg' will be added to the end of FILENAME
# when the file is uploaded.
#
# A timeout length of 20 seconds seems to work over most of my WAN links.
# For very slow connections (VPN from Colorado to Alaska) I sometimes use
# 120 seconds.
#
# For IOS devices, the running configuration is also written to NVRAM. That
# way, if you make changes and forget to write them to NVRAM, a power outage
# or end-user reboot won't leave you stranded.
#
# Example:
#
# Download running configuration of Cisco IOS device router-a at 172.16.0.1
# using SNMP community string 'private'. TFTP Server IP address is 172.16.0.254,
# and the timeout value is set to 60 seconds.
#
#   check_cisco_config_backup router-a 172.16.0.1 private ios 172.16.0.254 60
#

# Path to snmpset command.  This may have to be modified to match your
# system setup.
SNMPSET="/usr/local/bin/snmpset"

PROGNAME=`/bin/basename $0`
REVISION=`echo '$Revision: 1.17 $' | /bin/sed -e 's/[^0-9.]//g'`

# Grab the command line arguments
FILENAME=$1
HOST=$2
COMMUNITY=$3
OS=$4
SERVER=$5
TIMEOUT=$6

print_usage() {
    echo "Usage: $PROGNAME <filename> <host_ip> <snmp_community> <ios|catos> <tftp_server_ip> <timeout>"
    echo "Usage: $PROGNAME --help"
    echo "Usage: $PROGNAME --version"
}

print_help() {
    echo "$PROGNAME Ver. $REVISION"
    echo ""
    print_usage
    echo ""
    echo "Nagios Plugin: Backup Cisco configuration file to TFTP Server"
    echo ""
}

print_revision() {
    echo "$PROGNAME $REVISION"
}

# Cisco IOS devices use the OLD-CISCO-SYS-MIB
execute_ios() {
    $SNMPSET -t $TIMEOUT -c $COMMUNITY $HOST .1.3.6.1.4.1.9.2.1.55.$SERVER s $FILENAME.cfg
    $SNMPSET -t $TIMEOUT -c $COMMUNITY $HOST .1.3.6.1.4.1.9.2.1.54.0 i 1
}

# Cisco CatOS devices use the CISCO-STACK-MIB
execute_catos() {
    $SNMPSET -t $TIMEOUT -c $COMMUNITY $HOST .1.3.6.1.4.1.9.5.1.5.1.0 s $SERVER
    $SNMPSET -t $TIMEOUT -c $COMMUNITY $HOST .1.3.6.1.4.1.9.5.1.5.2.0 s $FILENAME.cfg
    $SNMPSET -t $TIMEOUT -c $COMMUNITY $HOST .1.3.6.1.4.1.9.5.1.5.3.0 i 1
    $SNMPSET -t $TIMEOUT -c $COMMUNITY $HOST .1.3.6.1.4.1.9.5.1.5.4.0 i 3
}

# Read output of execute functions, print message, return state
getfile() { 
    index=0
    if [ $cmd = "ios" ]; then
	while read "OUTPUT"; do
	    INPUT[$index]="$OUTPUT"
	    echo "${INPUT[$index]}"
	    index=$index+1
	done
	if [ "${INPUT[0]##*:}" = " \"$FILENAME.cfg\"" ] && [ "${INPUT[1]##*:}" = " 1" ]; then
	    echo "IOS Config file uploaded and written to NVRAM."
	    exit $STATE_OK
	elif [ "${INPUT[0]##*:}" = " \"$FILENAME.cfg\"" ] && [ "${INPUT[1]##*:}" != " 1" ]; then
	    echo "IOS Config file uploaded but not written to NVRAM."
	    exit $STATE_WARNING
	elif [ "${INPUT[0]%%:*}" = "Timeout" ]; then
	    echo "${INPUT[0]}"
	    exit $STATE_CRITICAL
	elif [ "${INPUT[0]}" = "Error in packet." ] && [ "${INPUT[1]%%)*}" = "Reason: (noSuchName"  ]; then
	    echo "${INPUT[0]} ${INPUT[1]}"
	    exit $STATE_CRITICAL
	elif [ "${INPUT[0]}" = "Error in packet." ] && [ "${INPUT[1]%%)*}" = "Reason: (genError"  ]; then
	    echo "${INPUT[0]} ${INPUT[1]}"
	    exit $STATE_CRITICAL
	else
	    echo "Error encountered!"
	    exit $STATE_UNKNOWN
	fi
    elif [ $cmd = "catos" ]; then
	while read "OUTPUT"; do
	    INPUT[$index]="$OUTPUT"
	    echo "${INPUT[$index]}"
	    index=$index+1
	done
	if [ "${INPUT[1]##*:}" = " \"$FILENAME.cfg\"" ]; then
	    echo "CatOS Config file uploaded."
	    exit $STATE_OK
	elif [ "${INPUT[1]%%:*}" = "Timeout" ]; then
	    echo "${INPUT[1]}"
	    exit $STATE_CRITICAL
	elif [ "${INPUT[0]}" = "Error in packet." ] && [ "${INPUT[1]%%)*}" = "Reason: (noSuchName"  ]; then
	    echo "${INPUT[0]} ${INPUT[1]}"
	    exit $STATE_CRITICAL
	elif [ "${INPUT[0]}" = "Error in packet." ] && [ "${INPUT[1]%%)*}" = "Reason: (genError"  ]; then
	    echo "${INPUT[0]} ${INPUT[1]}"
	    exit $STATE_CRITICAL
	else
	    echo "Error encountered!"
	    exit $STATE_UNKNOWN
	fi
    else 
	echo "$cmd not recognized as a valid OS."
    fi
}

# Make sure the correct number of command line
# arguments have been supplied

if [ $# -lt 6 ]; then
    print_usage
    exit $STATE_UNKNOWN
fi

exitstatus=$STATE_WARNING #default

case "$1" in
    --help|-h)
	print_help
	exit $STATE_OK
	;;
    --version|-V)
	print_revision
	exit $STATE_OK
	;;
esac

case "$OS" in
    ios)
	cmd="$OS" #execute_ios | getfile
	;;
    catos)
	cmd="$OS" #getfile $(execute_catos)
	;;
    *)
	echo "Unknown OS: $OS"
	print_usage
	exit $STATE_UNKNOWN
	;;
esac

execute_$cmd 2>&1 | getfile

exit $exitstatus
